Package: grouper
Type: Package
Title: Optimal Group Assignment and Workload Allocation
Version: 0.7.3
Authors@R: c(
    person("Vik", "Gopal", , "vik.gopal@nus.edu.sg", role = "aut"),
    person("Kevin", "Lam", , "kevinlam@nus.edu.sg", role = "aut"),
    person("Ju", "Xue", , "rigi.jx@gmail.com", role = "ctb"),
    person("Mingyuan", "Zhang", , "e0970135@u.nus.edu", role = c("aut", "cre")),
    person("National University of Singapore", role = "cph")
  )
Description: Integer programming models to assign students to groups by maximising
    diversity or topic preferences, and to allocate multi-role teaching workloads
    while balancing role demand, preferences, fairness, and cohort protection.
License: MIT + file LICENSE
URL: https://Zimmy313.github.io/grouper/,
        https://github.com/Zimmy313/grouper
BugReports: https://github.com/Zimmy313/grouper/issues
Encoding: UTF-8
LazyData: true
Suggests: knitr, ompr.roi, pkgdown, rmarkdown, ROI.plugin.glpk,
        ROI.plugin.highs, testthat (>= 3.0.0)
VignetteBuilder: knitr
Imports: cluster, dplyr, magrittr, ompr, rlang, yaml
Depends: R (>= 3.5)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
Config/roxygen2/markdown: TRUE
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-08 05:02:21 UTC; mingyuanzhang
Author: Vik Gopal [aut],
  Kevin Lam [aut],
  Ju Xue [ctb],
  Mingyuan Zhang [aut, cre],
  National University of Singapore [cph]
Maintainer: Mingyuan Zhang <e0970135@u.nus.edu>
Repository: CRAN
Date/Publication: 2026-07-08 09:00:02 UTC
Built: R 4.5.2; ; 2026-07-08 10:05:13 UTC; unix
