# Generated by roxygen2: do not edit by hand

export("%>%")
export(assign_groups)
export(assign_job)
export(compute_diversity)
export(convert_pref_mat)
export(extract_info)
export(extract_multirole_info)
export(extract_params_yaml)
export(extract_phd_info)
export(extract_student_info)
export(get_group_pref_score)
export(prepare_diversity_model)
export(prepare_model)
export(prepare_multirole_model)
export(prepare_phd_model)
export(prepare_preference_model)
export(solve_assignment)
export(summary_dba)
export(summary_pba)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
