
# grouper

<!-- badges: start -->
<!-- badges: end -->

The goal of grouper is to make it easier for an instructor to divide students
into groups.

## Installation

You can install the development version of grouper from [GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("singator/grouper")
```

But to be honest, the package on CRAN is already the most updated on. 
Please refer to the [vignettes](https://cran.r-project.org/package=grouper) for examples and documentation on the models in the package.

For best performance, it is best to get a license for [Gurobi optimiser](https://www.gurobi.com/downloads/).


## Contact

For more details, please contact Vik Gopal <vik.gopal@nus.edu.sg> or 
Kevin Lam <lamfy@u.nus.edu>.
